window.Gleam.initSearch([{"doc":"starflow","title":"starflow","content":"# Starflow ✨\n\n[![Package Version](https://img.shields.io/hexpm/v/flow)](https://hex.pm/packages/starflow)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/starflow/)\n\nStarflow is a Gleam library for building stateful chains of LLM interactions. It provides a simple, type-safe way to create flows that transform state into prompts, interact with language models, and parse responses back into state.\n\n&gt; [!NOTE]\n&gt; This library currently only supports Claude&#39;s send_message API\n\n## Installation\n\n```sh\ngleam add starflow\n```\n\n## Quick Start\n\nHere&#39;s a simple example that asks Claude to tell a joke:\n\n```gleam\nimport gleam/io\nimport gleam/result\nimport gleam/string\nimport envoy\n\nimport starflow\nimport starflow/api_key\nimport starflow/model\nimport starflow/providers\nimport starflow/state\n\npub fn main() {\n  let result = {\n    // Setup API key and model\n    use env_api_key &lt;- result.try(\n      envoy.get(&quot;ANTHROPIC_API_KEY&quot;)\n      |&gt; result.replace_error(&quot;api key not set!&quot;),\n    )\n    let api_key = api_key.new(providers.Anthropic, env_api_key)\n    let model = model.new(api_key)\n\n    // Define prompt transformer\n    let prompt = fn(state) {\n      [state.TextContent(&quot;Tell me a joke.&quot;)]\n    }\n\n    // Create and execute flow\n    let flow =\n      starflow.new(model)\n      |&gt; starflow.with_prompt(prompt)\n\n    let state = state.new(Nil)\n    starflow.invoke(state, flow)\n  }\n\n  case result {\n    Ok(state) -&gt; io.println(string.inspect(state))\n    Error(err) -&gt; io.println_error(err)\n  }\n}\n```\n\n## Key Features\n\n### State Management\n- Type-safe state that can hold any custom data\n- Automatic conversation history tracking\n- Token usage statistics\n\n### Transformers\n- Custom prompt generation from state\n- Flexible response parsing back to state\n- Default transformers for simple use cases\n\n### Provider Support\n- Currently supports Anthropic&#39;s Claude\n- Extensible provider system for future LLMs\n\n## Examples\n\nSee the [test directory](./test/starflow_test/) for examples!\n\n## Common Patterns\n\n### Simple Question-Answer\n```gleam\nlet flow =\n  starflow.new(model)\n  |&gt; starflow.with_prompt(fn(question) {\n    [state.TextContent(question)]\n  })\n\nstarflow.invoke(state.new(&quot;What is 2+2?&quot;), flow)\n```\n\n### Stateful Interactions\n```gleam\n// Define custom state type\npub type GameState {\n  GameState(target: Int, guesses: List(Int))\n}\n\n// Create game flow\nlet game_flow =\n  starflow.new(model)\n  |&gt; starflow.with_prompt(fn(state: GameState) {\n    [state.TextContent(&quot;Guess a number between 1 and &quot; &lt;&gt; int.to_string(state.target))]\n  })\n  |&gt; starflow.with_parser(fn(state, resp) {\n    // Parse response and update game state\n    let guess = parse_number(resp)\n    state.State(..state, any: GameState(..state.any, guesses: [guess, ..state.any.guesses]))\n  })\n```\n\n### Response Parsing\n```gleam\nlet flow =\n  starflow.new(model)\n  |&gt; starflow.with_parser(fn(state, resp) {\n    case resp.content {\n      [state.TextContent(text)] -&gt; {\n        // Process response text\n        state.State(..state, any: process_text(text))\n      }\n      _ -&gt; state\n    }\n  })\n```\n\n## Development\n\n```sh\ngleam run   # Run the project\n```\n\n## Coming Soon\n- Chain composition for complex flows\n- More sophisticated state management\n- Additional provider support\n- Streaming responses\n- Tool usage support\n\n## Documentation\n\nFor detailed documentation visit [hexdocs.pm/starflow](https://hexdocs.pm/starflow).\n","url":"index.html"},{"doc":"starflow/schema","title":"Property","content":"pub type Property {\n  Property(name: String, schema: Schema, required: Bool)\n}\n Represents an object property\nProperty(name: String, schema: Schema, required: Bool)\n\n\nSynonyms:\nstarflow/schema.Property\nstarflow/schema Property","url":"starflow/schema.html#Property"},{"doc":"starflow/schema","title":"Schema","content":"pub type Schema {\n  Schema(json: json.Json)\n}\n Represents a JSON Schema type\nSchema(json: json.Json)\n\n\nSynonyms:\nstarflow/schema.Schema\nstarflow/schema Schema","url":"starflow/schema.html#Schema"},{"doc":"starflow/schema","title":"array","content":"pub fn array(items: Schema) -&gt; Schema\n Creates an array type schema\nSynonyms:\nstarflow/schema.array\nstarflow/schema array","url":"starflow/schema.html#array"},{"doc":"starflow/schema","title":"boolean","content":"pub fn boolean() -&gt; Schema\n Creates a boolean type schema\nSynonyms:\nstarflow/schema.boolean\nstarflow/schema boolean","url":"starflow/schema.html#boolean"},{"doc":"starflow/schema","title":"enum","content":"pub fn enum(values: List(String)) -&gt; Schema\n Creates an enum type schema\nSynonyms:\nstarflow/schema.enum\nstarflow/schema enum","url":"starflow/schema.html#enum"},{"doc":"starflow/schema","title":"integer","content":"pub fn integer() -&gt; Schema\n Creates an integer type schema\nSynonyms:\nstarflow/schema.integer\nstarflow/schema integer","url":"starflow/schema.html#integer"},{"doc":"starflow/schema","title":"number","content":"pub fn number() -&gt; Schema\n Creates a number type schema\nSynonyms:\nstarflow/schema.number\nstarflow/schema number","url":"starflow/schema.html#number"},{"doc":"starflow/schema","title":"object","content":"pub fn object(properties: List(Property)) -&gt; Schema\n Creates an object type schema\nSynonyms:\nstarflow/schema.object\nstarflow/schema object","url":"starflow/schema.html#object"},{"doc":"starflow/schema","title":"optional","content":"pub fn optional(name: String, schema: Schema) -&gt; Property\n Creates an optional property\nSynonyms:\nstarflow/schema.optional\nstarflow/schema optional","url":"starflow/schema.html#optional"},{"doc":"starflow/schema","title":"required","content":"pub fn required(name: String, schema: Schema) -&gt; Property\n Creates a required property\nSynonyms:\nstarflow/schema.required\nstarflow/schema required","url":"starflow/schema.html#required"},{"doc":"starflow/schema","title":"string","content":"pub fn string() -&gt; Schema\n Creates a string type schema\nSynonyms:\nstarflow/schema.string\nstarflow/schema string","url":"starflow/schema.html#string"},{"doc":"starflow/schema","title":"string_with_format","content":"pub fn string_with_format(format: String) -&gt; Schema\n Creates a string type schema with format\nSynonyms:\nstarflow/schema.string_with_format\nstarflow/schema string_with_format","url":"starflow/schema.html#string_with_format"},{"doc":"starflow/schema","title":"starflow/schema","content":"","url":"starflow/schema.html"},{"doc":"starflow/tool","title":"Tool","content":"pub type Tool {\n  Tool(\n    name: String,\n    description: String,\n    schema: schema.Schema,\n    output: schema.Schema,\n    apply: fn(dynamic.Dynamic) -&gt;\n      Result(ToolResult, List(dynamic.DecodeError)),\n  )\n}\n Represents a tool that can be used by the model.\n The type parameter `input` defines the expected input type,\n while `output` defines the return type.\nTool(\n  name: String,\n  description: String,\n  schema: schema.Schema,\n  output: schema.Schema,\n  apply: fn(dynamic.Dynamic) -&gt;\n    Result(ToolResult, List(dynamic.DecodeError)),\n)\n\n\nSynonyms:\nstarflow/tool.Tool\nstarflow/tool Tool","url":"starflow/tool.html#Tool"},{"doc":"starflow/tool","title":"ToolResult","content":"pub type ToolResult {\n  String(String)\n  Number(Float)\n  Integer(Int)\n  Boolean(Bool)\n  Null\n  Array(List(ToolResult))\n  Object(List(#(String, ToolResult)))\n  Enum(String)\n}\n\nString(String)\n\n\nNumber(Float)\n\n\nInteger(Int)\n\n\nBoolean(Bool)\n\n\nNull\n\n\nArray(List(ToolResult))\n\n\nObject(List(#(String, ToolResult)))\n\n\nEnum(String)\n\n\nSynonyms:\nstarflow/tool.ToolResult\nstarflow/tool ToolResult","url":"starflow/tool.html#ToolResult"},{"doc":"starflow/tool","title":"result","content":"pub fn result(\n  tool_use: #(String, Dynamic),\n  tools: List(Tool),\n) -&gt; Result(ToolResult, DecodeError)\n\nSynonyms:\nstarflow/tool.result\nstarflow/tool result","url":"starflow/tool.html#result"},{"doc":"starflow/tool","title":"to_json","content":"pub fn to_json(tool: Tool) -&gt; Json\n Converts a tool definition to a JSON object matching the API specification\nSynonyms:\nstarflow/tool.to_json\nstarflow/tool to_json","url":"starflow/tool.html#to_json"},{"doc":"starflow/tool","title":"starflow/tool","content":"","url":"starflow/tool.html"},{"doc":"starflow/providers","title":"Provider","content":"pub type Provider {\n  Anthropic\n}\n Represents supported AI service providers.\n\n ## Variants\n - `Anthropic`: Provider for Claude and other Anthropic models\n\n ## Examples\n\n \n let provider = Anthropic\n let model = model.new(provider, api_key, &quot;claude-3-sonnet-20240229&quot;)\n \n\nAnthropic\n\n\nSynonyms:\nstarflow/providers.Provider\nstarflow/providers Provider","url":"starflow/providers.html#Provider"},{"doc":"starflow/providers","title":"url","content":"pub fn url(provider: Provider) -&gt; String\n Returns the base URL for the API of the specified provider.\n\n ## Examples\n\n \n url(Anthropic)  // Returns &quot;https://api.anthropic.com/v1&quot;\n \n\n ## Provider URLs\n - Anthropic: https://api.anthropic.com/v1\n\nSynonyms:\nstarflow/providers.url\nstarflow/providers url","url":"starflow/providers.html#url"},{"doc":"starflow/providers","title":"version","content":"pub fn version(provider: Provider) -&gt; String\n Returns the API version string required by the specified provider.\n\n This version identifier is used in API request headers to ensure\n compatibility with the provider&#39;s API.\n\n ## Examples\n\n \n version(Anthropic)  // Returns &quot;2023-06-01&quot;\n \n\n ## Provider Versions\n - Anthropic: 2023-06-01 (Claude API version)\n\nSynonyms:\nstarflow/providers.version\nstarflow/providers version","url":"starflow/providers.html#version"},{"doc":"starflow/providers","title":"starflow/providers","content":"","url":"starflow/providers.html"},{"doc":"starflow/api_key","title":"APIKey","content":"pub type APIKey {\n  APIKey(provider: providers.Provider, String)\n}\n Represents an API key associated with a specific provider.\n This type ensures that API keys are always paired with their corresponding provider,\n preventing misuse of keys across different services.\n\n ## Fields\n - `provider`: The provider this API key is for (e.g., Anthropic)\n - `String`: The actual API key value\n\n ## Examples\n\n \n // Create an Anthropic API key\n let key = new(providers.Anthropic, &quot;sk-ant-123...&quot;)\n\n // Pattern match if needed\n case key {\n   APIKey(providers.Anthropic, value) -&gt; // Handle Anthropic key\n   APIKey(other_provider, _) -&gt; // Handle other provider\n }\n \n\nAPIKey(provider: providers.Provider, String)\n\n\nSynonyms:\nstarflow/api_key.APIKey\nstarflow/api_key APIKey","url":"starflow/api_key.html#APIKey"},{"doc":"starflow/api_key","title":"get","content":"pub fn get(api_key: APIKey) -&gt; String\n Retrieves the raw API key string from an APIKey.\n This is typically used when making API requests where the raw key\n is needed for authentication.\n\n ## Examples\n\n \n let key = new(providers.Anthropic, &quot;sk-ant-123...&quot;)\n let raw_key = get(key)  // Returns &quot;sk-ant-123...&quot;\n\n // Common usage in request headers\n request.set_header(&quot;x-api-key&quot;, get(key))\n \n\nSynonyms:\nstarflow/api_key.get\nstarflow/api_key get","url":"starflow/api_key.html#get"},{"doc":"starflow/api_key","title":"new","content":"pub fn new(provider: Provider, key: String) -&gt; APIKey\n Creates a new APIKey with the specified provider and key value.\n\n ## Arguments\n - `provider`: The provider this key is associated with\n - `key`: The API key string\n\n ## Examples\n\n \n // Using environment variable\n use key &lt;- result.try(\n   envoy.get(&quot;ANTHROPIC_API_KEY&quot;)\n   |&gt; result.map(new(providers.Anthropic, _))\n )\n\n // Direct initialization\n let key = new(providers.Anthropic, &quot;sk-ant-123...&quot;)\n \n\nSynonyms:\nstarflow/api_key.new\nstarflow/api_key new","url":"starflow/api_key.html#new"},{"doc":"starflow/api_key","title":"starflow/api_key","content":"","url":"starflow/api_key.html"},{"doc":"starflow/state","title":"Content","content":"pub type Content {\n  TextContent(text: String)\n  ImageContent(source: ImageSource)\n  ToolContent(id: String, name: String, input: dynamic.Dynamic)\n}\n Represents different types of content that can be in a message.\n\n ## Variants\n - `TextContent`: Simple text content\n - `ImageContent`: Image data with source information\n - `ToolContent`: Tool invocation with input parameters\n\n ## Examples\n\n \n // Text content\n TextContent(&quot;Hello, world!&quot;)\n\n // Image content\n ImageContent(Base64Image(base64_data, &quot;image/jpeg&quot;))\n\n // Tool content\n ToolContent(\n   id: &quot;calc_123&quot;,\n   name: &quot;calculator&quot;,\n   input: dynamic.from_json(&quot;{\\&quot;operation\\&quot;: \\&quot;add\\&quot;, \\&quot;numbers\\&quot;: [1, 2]}&quot;),\n )\n \n\nTextContent(text: String)\n\n\nImageContent(source: ImageSource)\n\n\nToolContent(id: String, name: String, input: dynamic.Dynamic)\n\n\nSynonyms:\nstarflow/state.Content\nstarflow/state Content","url":"starflow/state.html#Content"},{"doc":"starflow/state","title":"ImageSource","content":"pub type ImageSource {\n  Base64Image(data: String, media_type: String)\n}\n Represents the source of an image in image content.\n\n ## Variants\n - `Base64Image`: Base64-encoded image data with media type\n\n ## Examples\n\n \n Base64Image(\n   data: &quot;iVBORw0KGgoAAAANSU...&quot;,\n   media_type: &quot;image/jpeg&quot;,\n )\n \n\nBase64Image(data: String, media_type: String)\n\n\nSynonyms:\nstarflow/state.ImageSource\nstarflow/state ImageSource","url":"starflow/state.html#ImageSource"},{"doc":"starflow/state","title":"Message","content":"pub type Message {\n  Message(role: String, content: List(Content))\n}\n Represents a message in the conversation.\n\n ## Fields\n - `role`: The role of the message sender (e.g., &quot;user&quot;, &quot;assistant&quot;)\n - `content`: List of content blocks in the message\n\n ## Examples\n\n \n // Simple text message\n Message(\n   role: &quot;user&quot;,\n   content: [TextContent(&quot;Hello!&quot;)],\n )\n\n // Message with image\n Message(\n   role: &quot;user&quot;,\n   content: [\n     TextContent(&quot;What&#39;s in this image?&quot;),\n     ImageContent(Base64Image(data, &quot;image/jpeg&quot;)),\n   ],\n )\n \n\nMessage(role: String, content: List(Content))\n\n\nSynonyms:\nstarflow/state.Message\nstarflow/state Message","url":"starflow/state.html#Message"},{"doc":"starflow/state","title":"Response","content":"pub type Response {\n  Response(\n    id: String,\n    content: List(Content),\n    model: String,\n    stop_reason: option.Option(StopReason),\n    usage: Usage,\n  )\n}\n Represents a response from the model.\n\n ## Fields\n - `id`: Unique identifier for the response\n - `content`: List of content blocks in the response\n - `model`: Name of the model that generated the response\n - `stop_reason`: Why the model stopped generating\n - `usage`: Token usage statistics for this response\n\nResponse(\n  id: String,\n  content: List(Content),\n  model: String,\n  stop_reason: option.Option(StopReason),\n  usage: Usage,\n)\n\n\nSynonyms:\nstarflow/state.Response\nstarflow/state Response","url":"starflow/state.html#Response"},{"doc":"starflow/state","title":"State","content":"pub type State(any) {\n  State(messages: List(Message), usage: Usage, any: any)\n}\n Represents the current state of a chain, including conversation history,\n token usage statistics, and custom state data.\n\n ## Fields\n - `messages`: List of messages in the conversation (TODO: should be held in reverse)\n - `usage`: Token usage statistics for the conversation\n - `any`: Custom state data that can be of any type\n\n ## Examples\n\n \n // Simple counter state\n type Counter {\n   Counter(value: Int)\n }\n let state = new(Counter(0))\n\n // Game state\n type GameState {\n   GameState(target: Int, guesses: List(Int))\n }\n let state = new(GameState(target: 42, guesses: []))\n \n\nState(messages: List(Message), usage: Usage, any: any)\n\n\nSynonyms:\nstarflow/state.State\nstarflow/state State","url":"starflow/state.html#State"},{"doc":"starflow/state","title":"StopReason","content":"pub type StopReason {\n  EndTurn\n  MaxTokens\n  StopSequence\n  ToolUse\n}\n Represents the reason why the model stopped generating output.\n\n ## Variants\n - `EndTurn`: Natural completion of the response\n - `MaxTokens`: Reached maximum token limit\n - `StopSequence`: Encountered a stop sequence\n - `ToolUse`: Stopped to use a tool\n\nEndTurn\n\n\nMaxTokens\n\n\nStopSequence\n\n\nToolUse\n\n\nSynonyms:\nstarflow/state.StopReason\nstarflow/state StopReason","url":"starflow/state.html#StopReason"},{"doc":"starflow/state","title":"Usage","content":"pub type Usage {\n  Usage(\n    input_tokens: Int,\n    cache_creation_input_tokens: option.Option(Int),\n    cache_read_input_tokens: option.Option(Int),\n    output_tokens: Int,\n  )\n}\n Tracks token usage statistics for the conversation.\n\n ## Fields\n - `input_tokens`: Number of tokens in the input\n - `cache_creation_input_tokens`: Optional tokens used in cache creation\n - `cache_read_input_tokens`: Optional tokens read from cache\n - `output_tokens`: Number of tokens in the output\n\nUsage(\n  input_tokens: Int,\n  cache_creation_input_tokens: option.Option(Int),\n  cache_read_input_tokens: option.Option(Int),\n  output_tokens: Int,\n)\n\n\nSynonyms:\nstarflow/state.Usage\nstarflow/state Usage","url":"starflow/state.html#Usage"},{"doc":"starflow/state","title":"new","content":"pub fn new(any: a) -&gt; State(a)\n Creates a new state with empty message history, zero usage, and the provided\n custom state data.\n\n ## Examples\n\n \n // Initialize with a string\n let state = new(&quot;initial data&quot;)\n\n // Initialize with a custom type\n type GameState { GameState(score: Int) }\n let state = new(GameState(0))\n \n\nSynonyms:\nstarflow/state.new\nstarflow/state new","url":"starflow/state.html#new"},{"doc":"starflow/state","title":"starflow/state","content":"","url":"starflow/state.html"},{"doc":"starflow/decoder","title":"response","content":"pub fn response(\n  body: Dynamic,\n) -&gt; Result(Response, List(DecodeError))\n\nSynonyms:\nstarflow/decoder.response\nstarflow/decoder response","url":"starflow/decoder.html#response"},{"doc":"starflow/decoder","title":"starflow/decoder","content":"","url":"starflow/decoder.html"},{"doc":"starflow/model","title":"MaxTokens","content":"pub type MaxTokens =\n  Int\n Type alias for maximum tokens allowed in model response.\n\n ## Notes\n - Higher values allow for longer responses but use more tokens\n - Provider-specific limits apply\n - Consider cost implications when setting this value\n\n\nSynonyms:\nstarflow/model.MaxTokens\nstarflow/model MaxTokens","url":"starflow/model.html#MaxTokens"},{"doc":"starflow/model","title":"Model","content":"pub type Model {\n  Model(\n    provider: providers.Provider,\n    api_key: api_key.APIKey,\n    name: Name,\n    temperature: Temperature,\n    max_tokens: MaxTokens,\n  )\n}\n Configuration for a language model, including provider details and generation parameters.\n\n ## Fields\n - `provider`: The AI provider (e.g., Anthropic)\n - `api_key`: Authentication key for the provider\n - `name`: Specific model identifier\n - `temperature`: Controls output randomness (0.0 - 1.0)\n - `max_tokens`: Maximum tokens in model response\n\n ## Examples\n\n \n // Default Anthropic configuration\n let model = new(anthropic_api_key)\n\n // Custom configuration\n let model = new(api_key)\n   |&gt; with_name(&quot;claude-3-opus-20241022&quot;)\n   |&gt; with_temperature(0.9)\n \n\nModel(\n  provider: providers.Provider,\n  api_key: api_key.APIKey,\n  name: Name,\n  temperature: Temperature,\n  max_tokens: MaxTokens,\n)\n\n\nSynonyms:\nstarflow/model.Model\nstarflow/model Model","url":"starflow/model.html#Model"},{"doc":"starflow/model","title":"Name","content":"pub type Name =\n  String\n Type alias for model names. Used by providers to identify specific model versions.\n\n ## Examples\n - `&quot;claude-3-5-sonnet-20241022&quot;` for Anthropic\n\n\nSynonyms:\nstarflow/model.Name\nstarflow/model Name","url":"starflow/model.html#Name"},{"doc":"starflow/model","title":"Temperature","content":"pub type Temperature =\n  Float\n Type alias for model temperature setting. Controls randomness in model outputs.\n\n ## Range\n - 0.0: Most deterministic, best for analytical tasks\n - 1.0: Most random, best for creative tasks\n - Typical values: 0.0 - 1.0\n\n\nSynonyms:\nstarflow/model.Temperature\nstarflow/model Temperature","url":"starflow/model.html#Temperature"},{"doc":"starflow/model","title":"new","content":"pub fn new(api_key: APIKey) -&gt; Model\n Creates a new model configuration with provider-specific defaults.\n\n Currently supported providers:\n - Anthropic:\n   - Model: &quot;claude-3-5-sonnet-20241022&quot;\n   - Temperature: 0.7\n   - Max Tokens: 1024\n\n ## Examples\n\n \n let api_key = api_key.new(providers.Anthropic, &quot;your-key-here&quot;)\n let model = new(api_key)\n \n\nSynonyms:\nstarflow/model.new\nstarflow/model new","url":"starflow/model.html#new"},{"doc":"starflow/model","title":"with_name","content":"pub fn with_name(model: Model, name: String) -&gt; Model\n Updates the model name while preserving other settings.\n\n ## Examples\n\n \n // Switch to a different Claude model\n let model = new(api_key)\n   |&gt; with_name(&quot;claude-3-opus-20241022&quot;)\n\n // Use a specific model version\n let model = new(api_key)\n   |&gt; with_name(&quot;claude-3-5-sonnet-20240229&quot;)\n \n\n ## Notes\n - Ensure the model name is valid for the provider\n - Different models may have different capabilities and costs\n\nSynonyms:\nstarflow/model.with_name\nstarflow/model with_name","url":"starflow/model.html#with_name"},{"doc":"starflow/model","title":"starflow/model","content":"","url":"starflow/model.html"},{"doc":"starflow/encoder","title":"encode","content":"pub fn encode(\n  model: Model,\n  messages: List(Message),\n  tools: List(Tool),\n) -&gt; Json\n\nSynonyms:\nstarflow/encoder.encode\nstarflow/encoder encode","url":"starflow/encoder.html#encode"},{"doc":"starflow/encoder","title":"starflow/encoder","content":"","url":"starflow/encoder.html"},{"doc":"starflow/api_request","title":"APIError","content":"pub type APIError {\n  NetworkError(String)\n  DecodingError\n  InvalidResponse(Int, String)\n}\n Represents possible errors that can occur during API requests.\n\n ## Variants\n - `NetworkError`: Connection or HTTP-related errors with error message\n - `DecodingError`: Failed to decode request URL or response body\n - `InvalidResponse`: Received non-200 status code with status and body\n\n ## Examples\n\n \n case create_message(model, messages) {\n   Error(NetworkError(msg)) -&gt; // Handle network failure\n   Error(DecodingError) -&gt; // Handle malformed response\n   Error(InvalidResponse(status, body)) -&gt; // Handle API error response\n   Ok(response) -&gt; // Handle successful response\n }\n \n\nNetworkError(String)\n\n\nDecodingError\n\n\nInvalidResponse(Int, String)\n\n\nSynonyms:\nstarflow/api_request.APIError\nstarflow/api_request APIError","url":"starflow/api_request.html#APIError"},{"doc":"starflow/api_request","title":"create_message","content":"pub fn create_message(\n  model: Model,\n  messages: List(Message),\n  tools: List(Tool),\n) -&gt; Result(Response, APIError)\n Creates a new message by sending a request to the model&#39;s API.\n\n This function:\n 1. Builds the request URL based on the provider\n 2. Adds necessary headers\n 3. Encodes the messages into the request body\n 4. Sends the request\n 5. Decodes the response\n\n ## Arguments\n - `model`: The model configuration to use\n - `messages`: List of messages to send to the model\n\n ## Returns\n A Result containing either:\n - `Ok(Response)`: The successful API response\n - `Error(APIError)`: An error that occurred during the request\n\n ## Examples\n\n \n // Simple message creation\n let messages = [\n   state.Message(\n     role: &quot;user&quot;,\n     content: [state.TextContent(&quot;Hello!&quot;)],\n   )\n ]\n\n case create_message(model, messages) {\n   Ok(response) -&gt; handle_response(response)\n   Error(error) -&gt; handle_error(error)\n }\n\n // With image content\n let messages = [\n   state.Message(\n     role: &quot;user&quot;,\n     content: [\n       state.TextContent(&quot;What&#39;s in this image?&quot;),\n       state.ImageContent(source: image_source),\n     ],\n   )\n ]\n\n create_message(model, messages)\n \n\nSynonyms:\nstarflow/api_request.create_message\nstarflow/api_request create_message","url":"starflow/api_request.html#create_message"},{"doc":"starflow/api_request","title":"starflow/api_request","content":"","url":"starflow/api_request.html"},{"doc":"starflow/transform","title":"Parser","content":"pub type Parser(any) =\n  fn(\n    state.State(any),\n    state.Response,\n    List(#(String, tool.ToolResult)),\n  ) -&gt;\n    state.State(any)\n A function that updates the chain&#39;s state based on the model&#39;s response.\n This allows for custom processing of the model&#39;s output and updating the\n state accordingly.\n\n ## Examples\n\n \n let update_game = fn(state, response) {\n   // Extract guess from response\n   let guess = parse_guess(response.content)\n   // Update game state with new guess\n   state.State(..state, guesses: [guess, ..state.guesses])\n }\n\n let accumulate_summary = fn(state, response) {\n   // Add response to list of summaries\n   state.State(..state, summaries: [response.content, ..state.summaries])\n }\n \n\n\nSynonyms:\nstarflow/transform.Parser\nstarflow/transform Parser","url":"starflow/transform.html#Parser"},{"doc":"starflow/transform","title":"Prompt","content":"pub type Prompt(any) =\n  fn(state.State(any)) -&gt; List(state.Content)\n A function that transforms the chain&#39;s state into a list of content blocks\n that will be sent to the model as a prompt.\n\n ## Examples\n\n \n let simple_prompt = fn(state) {\n   [state.TextContent(&quot;Hello!&quot;)]\n }\n\n let template_prompt = fn(data: TemplateData) {\n   [state.TextContent(&quot;User name: &quot; &lt;&gt; data.name)]\n }\n \n\n\nSynonyms:\nstarflow/transform.Prompt\nstarflow/transform Prompt","url":"starflow/transform.html#Prompt"},{"doc":"starflow/transform","title":"parser_default","content":"pub fn parser_default(\n  state: State(a),\n  last_response: Response,\n  tool_results: List(#(String, ToolResult)),\n) -&gt; State(a)\n Default parser transformer function that:\n 1. Preserves the message history by appending the model&#39;s response\n 2. Updates the usage information\n 3. Maintains the existing state\n\n This is useful when you just want to accumulate the conversation history\n without any special processing of the responses.\n\n ## Examples\n\n \n let flow =\n   starflow.new(model)\n   |&gt; starflow.with_parser(parser_default)\n\n // Each response will be added to the message history\n // and usage stats will be updated\n \n\nSynonyms:\nstarflow/transform.parser_default\nstarflow/transform parser_default","url":"starflow/transform.html#parser_default"},{"doc":"starflow/transform","title":"prompt_default","content":"pub fn prompt_default(state: State(a)) -&gt; List(Content)\n Default prompt transformer that simply inspects the state and wraps it\n in a TextContent block.\n\n ## Examples\n\n \n let state = &quot;Hello&quot;\n prompt_default(state)  // Returns [TextContent(&quot;\\&quot;Hello\\&quot;&quot;)]\n\n let state = 42\n prompt_default(state)  // Returns [TextContent(&quot;42&quot;)]\n \n\nSynonyms:\nstarflow/transform.prompt_default\nstarflow/transform prompt_default","url":"starflow/transform.html#prompt_default"},{"doc":"starflow/transform","title":"starflow/transform","content":"","url":"starflow/transform.html"},{"doc":"starflow","title":"Flow","content":"pub type Flow(any) {\n  Flow(\n    model: model.Model,\n    prompt: transform.Prompt(any),\n    parser: transform.Parser(any),\n    tools: List(tool.Tool),\n  )\n}\n Represents a flow of transformations from state to prompt to response and back to state.\n\n ## Fields\n - `model`: The model configuration for API requests\n - `prompt`: Function to transform state into a prompt\n - `parser`: Function to transform response back into state\n\n ## Example\n\n \n // Create a flow for a number guessing game\n type GameState {\n   GameState(target: Int, guesses: List(Int))\n }\n\n let game_flow =\n   new(model)\n   |&gt; with_prompt(fn(state) {\n     [state.TextContent(&quot;Guess a number between 1 and 10&quot;)]\n   })\n   |&gt; with_parser(fn(state, resp) {\n     // Parse response and update game state\n     let guess = extract_guess(resp)\n     let new_guesses = [guess, ..state.any.guesses]\n     state.State(..state, any: GameState(..state.any, guesses: new_guesses))\n   })\n \n\nFlow(\n  model: model.Model,\n  prompt: transform.Prompt(any),\n  parser: transform.Parser(any),\n  tools: List(tool.Tool),\n)\n\n\nSynonyms:\nstarflow.Flow\nstarflow Flow","url":"starflow.html#Flow"},{"doc":"starflow","title":"invoke","content":"pub fn invoke(\n  state: State(a),\n  flow: Flow(a),\n) -&gt; Result(State(a), APIError)\n Executes one step of the Flow:\n 1. Transforms state into a prompt using the prompt transformer\n 2. Sends the prompt to the model\n 3. Transforms the response back into state using the parser\n\n ## Examples\n\n \n // Simple counter flow\n let counter_flow =\n   new(model)\n   |&gt; with_prompt(fn(count) {\n     [state.TextContent(&quot;Add one to &quot; &lt;&gt; int.to_string(count))]\n   })\n   |&gt; with_parser(fn(state, resp) {\n     // Parse response to get new count\n     let new_count = state.any + 1\n     state.State(..state, any: new_count)\n   })\n\n // Run one step\n case invoke(state.new(0), counter_flow) {\n   Ok(new_state) -&gt; // Handle updated state\n   Error(error) -&gt; // Handle API error\n }\n \n\nSynonyms:\nstarflow.invoke\nstarflow invoke","url":"starflow.html#invoke"},{"doc":"starflow","title":"new","content":"pub fn new(model: Model) -&gt; Flow(a)\n Creates a new Flow with the given model and default transformers.\n\n ## Examples\n\n \n let flow = new(model)\n // Creates a flow with:\n // - The specified model\n // - Default prompt transformer (string inspection)\n // - Default response parser (message accumulation)\n \n\nSynonyms:\nstarflow.new\nstarflow new","url":"starflow.html#new"},{"doc":"starflow","title":"with_model","content":"pub fn with_model(flow: Flow(a), model: Model) -&gt; Flow(a)\n Updates the model configuration of a Flow.\n\n ## Examples\n\n \n let updated_flow =\n   flow\n   |&gt; with_model(new_model)\n \n\nSynonyms:\nstarflow.with_model\nstarflow with_model","url":"starflow.html#with_model"},{"doc":"starflow","title":"with_parser","content":"pub fn with_parser(\n  flow: Flow(a),\n  parser: fn(State(a), Response, List(#(String, ToolResult))) -&gt;\n    State(a),\n) -&gt; Flow(a)\n Sets a custom response parser for the Flow.\n\n ## Examples\n\n \n let flow =\n   new(model)\n   |&gt; with_parser(fn(state, resp) {\n     // Extract and process the response\n     case resp.content {\n       [state.TextContent(text)] -&gt; {\n         let count = parse_number(text)\n         state.State(..state, any: count)\n       }\n       _ -&gt; state\n     }\n   })\n \n\nSynonyms:\nstarflow.with_parser\nstarflow with_parser","url":"starflow.html#with_parser"},{"doc":"starflow","title":"with_prompt","content":"pub fn with_prompt(\n  flow: Flow(a),\n  prompt: fn(State(a)) -&gt; List(Content),\n) -&gt; Flow(a)\n Sets a custom prompt transformer for the Flow.\n\n ## Examples\n\n \n let flow =\n   new(model)\n   |&gt; with_prompt(fn(state) {\n     [state.TextContent(&quot;Current count: &quot; &lt;&gt; int.to_string(state))]\n   })\n \n\nSynonyms:\nstarflow.with_prompt\nstarflow with_prompt","url":"starflow.html#with_prompt"},{"doc":"starflow","title":"with_tool","content":"pub fn with_tool(flow: Flow(a), tool: Tool) -&gt; Flow(a)\n\nSynonyms:\nstarflow.with_tool\nstarflow with_tool","url":"starflow.html#with_tool"},{"doc":"starflow","title":"starflow","content":"","url":"starflow.html"},{"doc":"tools/calculator","title":"tool","content":"pub fn tool() -&gt; Tool\n\nSynonyms:\ntools/calculator.tool\ntools/calculator tool","url":"tools/calculator.html#tool"},{"doc":"tools/calculator","title":"tools/calculator","content":"","url":"tools/calculator.html"}]);